;; -*- no-byte-compile: t; lexical-binding: nil -*-
(define-package "advance-words-count" "0.8.8.0.20170405.2"
  "A Package Provides Extended `count-words'."
  '((pos-tip "0"))
  :url "https://github.com/LdBeth/advance-words-count.el/"
  :commit "9668d642e9411eb88b1b77b72f6825b6a01f3e84"
  :revdesc "9668d642e941"
  :keywords '("wp")
  :authors '(("LdBeth" . "andpuke@foxmail.com"))
  :maintainers '(("LdBeth" . "andpuke@foxmail.com")))
